@@ -22,19 +22,22 @@ set -ex
 arrow_dir=${1}
 build_dir=${2}
 
-
+# We don't need to follow this external file.
+# See also: https://www.shellcheck.net/wiki/SC1090
+#
+# shellcheck source=/dev/null
 source ~/emsdk/emsdk_env.sh
 
 source_dir=${arrow_dir}/python
 python_build_dir=${build_dir}/python
 
-rm -rf ${python_build_dir}
-cp -aL ${source_dir} ${python_build_dir}
+rm -rf "${python_build_dir}"
+cp -aL "${source_dir}" "${python_build_dir}"
 
 # conda sets LDFLAGS / CFLAGS etc. which break
 # emcmake so we unset them
 unset LDFLAGS CFLAGS CXXFLAGS CPPFLAGS
 
-pushd ${python_build_dir}
+pushd "${python_build_dir}"
 pyodide build
 popd
