@@ -23,6 +23,7 @@
 #include <unordered_map>
 #include <unordered_set>
 
+#include "arrow/acero/exec_plan_internal.h"
 #include "arrow/acero/options.h"
 #include "arrow/acero/query_context.h"
 #include "arrow/acero/task_util.h"
@@ -38,7 +39,7 @@
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/io_util.h"
 #include "arrow/util/key_value_metadata.h"
-#include "arrow/util/logging.h"
+#include "arrow/util/logging_internal.h"
 #include "arrow/util/string.h"
 #include "arrow/util/tracing_internal.h"
 #include "arrow/util/vector.h"
@@ -1101,23 +1102,6 @@ Result<std::unique_ptr<RecordBatchReader>> DeclarationToReader(
   return DeclarationToReader(std::move(declaration), std::move(options));
 }
 
-namespace internal {
-
-void RegisterSourceNode(ExecFactoryRegistry*);
-void RegisterFetchNode(ExecFactoryRegistry*);
-void RegisterFilterNode(ExecFactoryRegistry*);
-void RegisterOrderByNode(ExecFactoryRegistry*);
-void RegisterPivotLongerNode(ExecFactoryRegistry*);
-void RegisterProjectNode(ExecFactoryRegistry*);
-void RegisterUnionNode(ExecFactoryRegistry*);
-void RegisterAggregateNode(ExecFactoryRegistry*);
-void RegisterSinkNode(ExecFactoryRegistry*);
-void RegisterHashJoinNode(ExecFactoryRegistry*);
-void RegisterAsofJoinNode(ExecFactoryRegistry*);
-void RegisterSortedMergeNode(ExecFactoryRegistry*);
-
-}  // namespace internal
-
 ExecFactoryRegistry* default_exec_factory_registry() {
   class DefaultRegistry : public ExecFactoryRegistry {
    public:
