@@ -20,6 +20,7 @@
 #include <chrono>
 #include <mutex>
 #include <optional>
+#include <thread>
 #include <vector>
 
 #include "arrow/array/builder_primitive.h"
@@ -232,7 +233,7 @@ class DatasetWriterTestFixture : public testing::Test {
   util::AsyncTaskScheduler* scheduler_;
   Future<> scheduler_finished_;
   FileSystemDatasetWriteOptions write_options_;
-  bool paused_{false};
+  std::atomic_bool paused_{false};
   uint64_t counter_ = 0;
 };
 
@@ -275,6 +276,41 @@ TEST_F(DatasetWriterTestFixture, BatchGreaterThanMaxRowsQueued) {
   ASSERT_EQ(paused_, false);
 }
 
+TEST_F(DatasetWriterTestFixture, BatchWriteConcurrent) {
+#ifndef ARROW_ENABLE_THREADING
+  GTEST_SKIP() << "Test requires threading support";
+#endif
+  auto dataset_writer = MakeDatasetWriter(/*max_rows=*/5);
+
+  for (int threads = 1; threads < 5; threads++) {
+    for (int iter = 2; iter <= 256; iter *= 2) {
+      for (int batch = 2; batch <= 64; batch *= 2) {
+        std::vector<std::thread> workers;
+        for (int i = 0; i < threads; ++i) {
+          workers.push_back(std::thread([&, i = i]() {
+            for (int j = 0; j < iter; ++j) {
+              while (paused_) {
+                SleepABit();
+              }
+              dataset_writer->WriteRecordBatch(MakeBatch(0, batch + i + 10 * j), "");
+            }
+          }));
+        }
+        for (std::thread& t : workers) {
+          if (t.joinable()) {
+            t.join();
+          }
+          while (paused_) {
+            SleepABit();
+          }
+        }
+      }
+    }
+  }
+  EndWriterChecked(dataset_writer.get());
+  ASSERT_EQ(paused_, false);
+}
+
 TEST_F(DatasetWriterTestFixture, MaxRowsOneWrite) {
   write_options_.max_rows_per_file = 10;
   write_options_.max_rows_per_group = 10;
