@@ -352,6 +352,21 @@ test_that("array supports integer64", {
   expect_true(as.vector(is.na(all_na)))
 })
 
+test_that("array supports integer64 with new semantics", {
+  withr::with_options(list(integer64_semantics = "new"), {
+    x <- bit64::as.integer64(1:10) + MAX_INT
+    expect_array_roundtrip(x, int64())
+
+    x[4] <- NA
+    expect_array_roundtrip(x, int64())
+
+    # all NA int64 (ARROW-3795)
+    all_na <- arrow_array(bit64::as.integer64(NA))
+    expect_type_equal(all_na, int64())
+    expect_true(as.vector(is.na(all_na)))
+  })
+})
+
 test_that("array supports hms difftime", {
   time <- hms::hms(56, 34, 12)
   expect_array_roundtrip(c(time, time), time32("s"))
@@ -800,7 +815,7 @@ test_that("arrow_array() handles vector -> fixed size list arrays", {
 })
 
 test_that("Handling string data with embedded nuls", {
-  raws <- structure(
+  raws <- blob::as_blob(
     list(
       as.raw(c(0x70, 0x65, 0x72, 0x73, 0x6f, 0x6e)),
       as.raw(c(0x77, 0x6f, 0x6d, 0x61, 0x6e)),
@@ -808,8 +823,7 @@ test_that("Handling string data with embedded nuls", {
       as.raw(c(0x66, 0x00, 0x00, 0x61, 0x00, 0x6e)), # multiple nuls
       as.raw(c(0x63, 0x61, 0x6d, 0x65, 0x72, 0x61)),
       as.raw(c(0x74, 0x76))
-    ),
-    class = c("arrow_binary", "vctrs_vctr", "list")
+    )
   )
   expect_error(
     rawToChar(raws[[3]]),
@@ -1261,7 +1275,7 @@ test_that("concat_arrays works", {
 
   concat_int <- concat_arrays(arrow_array(1:3), arrow_array(4:5))
   expect_true(concat_int$type == int32())
-  expect_equal(concat_int,  arrow_array(1:5))
+  expect_equal(concat_int, arrow_array(1:5))
 
   concat_int64 <- concat_arrays(
     arrow_array(1:3),
@@ -1322,7 +1336,14 @@ test_that("Array to C-interface", {
 })
 
 test_that("Can convert R integer/double to decimal (ARROW-11631)", {
-  # Check both decimal128 and decimal256
+  # Check all of decimal32, decimal64, decimal128 and decimal256
+
+
+  decimal32_from_dbl <- arrow_array(c(1, NA_real_), type = decimal32(9, 2))
+  decimal64_from_dbl <- arrow_array(c(1, NA_real_), type = decimal64(12, 2))
+  decimal32_from_int <- arrow_array(c(1L, NA_integer_), type = decimal32(9, 2))
+  decimal64_from_int <- arrow_array(c(1L, NA_integer_), type = decimal64(12, 2))
+
   decimal128_from_dbl <- arrow_array(c(1, NA_real_), type = decimal128(12, 2))
   decimal256_from_dbl <- arrow_array(c(1, NA_real_), type = decimal256(12, 2))
   decimal128_from_int <- arrow_array(c(1L, NA_integer_), type = decimal128(12, 2))
@@ -1334,6 +1355,16 @@ test_that("Can convert R integer/double to decimal (ARROW-11631)", {
   decimal_from_altrep_dbl <- arrow_array(altrep_dbl, type = decimal128(12, 2))
   decimal_from_altrep_int <- arrow_array(altrep_int, type = decimal128(12, 2))
 
+  expect_equal(
+    decimal32_from_dbl,
+    arrow_array(c(1, NA))$cast(decimal32(9, 2))
+  )
+
+  expect_equal(
+    decimal64_from_dbl,
+    arrow_array(c(1, NA))$cast(decimal64(12, 2))
+  )
+
   expect_equal(
     decimal128_from_dbl,
     arrow_array(c(1, NA))$cast(decimal128(12, 2))
@@ -1344,6 +1375,17 @@ test_that("Can convert R integer/double to decimal (ARROW-11631)", {
     arrow_array(c(1, NA))$cast(decimal256(12, 2))
   )
 
+  expect_equal(
+    decimal32_from_int,
+    arrow_array(c(1, NA))$cast(decimal32(9, 2))
+  )
+
+  expect_equal(
+    decimal64_from_int,
+    arrow_array(c(1, NA))$cast(decimal64(12, 2))
+  )
+
+
   expect_equal(
     decimal128_from_int,
     arrow_array(c(1, NA))$cast(decimal128(12, 2))
@@ -1370,3 +1412,10 @@ test_that("Can convert R integer/double to decimal (ARROW-11631)", {
     "Conversion to decimal from non-integer/double"
   )
 })
+
+test_that("Array handles negative fractional dates correctly (GH-46873)", {
+  # `origin` must be specified for compatibility with R versions < 4.2.0
+  d <- as.Date(-0.1, origin = "1970-01-01")
+  arr <- arrow_array(d)
+  expect_equal(as.vector(arr), as.Date("1969-12-31", origin = "1970-01-01"))
+})
